// Copyright (C) 2020 ~ 2021, Deepin Technology Co., Ltd. <support@deepin.org>
// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include <DSettings>

void GenerateSettingTranslate()
{
    auto base_play_addsimilarText = QObject::tr("Auto add similar files to play");
    auto base_play_emptylistText = QObject::tr("Clear playlist when exit");
    auto base_play_showthumbnailmode = QObject::tr("Show thumbnails in progress bar");
    auto base_play_mousepreviewText = QObject::tr("Show video preview on mouseover");
    auto base_play_multiinstanceText = QObject::tr("Open a new player for each file played");
    auto base_play_pauseonminText = QObject::tr("Pause when minimized");
    auto base_play_resumelastText = QObject::tr("Remember playback position");
    auto base_play_defaultplaymode = QObject::tr("Default play mode");
    auto base_screenshot_locationName = QObject::tr("Path");
    auto group_baseName = QObject::tr("Basic");
    auto group_base_playName = QObject::tr("Play");
    auto group_base_screenshotName = QObject::tr("Screenshot");
    auto group_shortcutsName = QObject::tr("Shortcuts");
    auto group_shortcuts_fileName = QObject::tr("File");
    auto group_shortcuts_frame_soundName = QObject::tr("Frame/Sound");
    auto group_shortcuts_playName = QObject::tr("Playback");
    auto group_shortcuts_screenshotName = QObject::tr("Screenshot");
    auto group_shortcuts_subName = QObject::tr("Subtitle");
    auto group_subtitleName = QObject::tr("Subtitle");
    auto group_subtitle_fontName = QObject::tr("Font Style");
    auto reset_button_name = QObject::tr("Restore Defaults");
    auto shortcuts_file_open_fileName = QObject::tr("Open file");
    auto shortcuts_file_playlist_nextName = QObject::tr("Open next");
    auto shortcuts_file_playlist_prevName = QObject::tr("Open previous");
    auto shortcuts_frame_sound_miniName = QObject::tr("Mini mode");
    auto shortcuts_frame_sound_muteName = QObject::tr("Mute");
    auto shortcuts_frame_sound_next_frameName = QObject::tr("Next frame");
    auto shortcuts_frame_sound_previous_frameName = QObject::tr("Previous frame");
    auto shortcuts_frame_sound_vol_downName = QObject::tr("Volume down");
    auto shortcuts_frame_sound_vol_upName = QObject::tr("Volume up");
    auto shortcuts_play_accelName = QObject::tr("Speed up");
    auto shortcuts_play_decelName = QObject::tr("Speed down");
    auto shortcuts_play_fullscreenName = QObject::tr("Fullscreen");
    auto shortcuts_play_pause_playName = QObject::tr("Pause/Play");
    auto shortcuts_play_playlistName = QObject::tr("Playlist");
    auto shortcuts_play_resetName = QObject::tr("Reset speed");
    auto shortcuts_play_deleteOne = QObject::tr("Delete from playlist");
    auto shortcuts_play_movieInfo = QObject::tr("Film info");
    auto shortcuts_play_seek_backwardName = QObject::tr("Rewind");
    auto shortcuts_play_seek_forwardName = QObject::tr("Forward");
    auto shortcuts_screenshot_burst_screenshotName = QObject::tr("Burst shooting");
    auto shortcuts_screenshot_screenshotName = QObject::tr("Film screenshot");
    auto shortcuts_sub_sub_backwardName = QObject::tr("0.5s backward");
    auto shortcuts_sub_sub_forwardName = QObject::tr("0.5s forward");
    auto subtitle_font_familyName = QObject::tr("Font");
    auto subtitle_font_sizeName = QObject::tr("Size");
    auto group_shortcuts_fullscreen = QObject::tr("Exit fullscreen/mini mode");
    auto group_shortcuts_help = QObject::tr("Help");
    auto group_shortcuts_showshortcuts = QObject::tr("Display shortcuts");
    auto group_settings = QObject::tr("Settings");
    auto group_decoding = QObject::tr("Decoding");
    auto group_decoding_auto = QObject::tr("Auto select (Recommended)");
    auto group_decoding_hardware = QObject::tr("Hardware decoding");
    auto group_decoding_software = QObject::tr("Software decoding");
    auto group_decoding_customize = QObject::tr("Customize");
    auto group_customize_Decodemode = QObject::tr("Decode mode");
    auto group_customize_Videoout = QObject::tr("Video out");
    auto group_effectmode_Effectmode = QObject::tr("Rendering engine");
}

