/****************************************************************************
** Meta object code from reading C++ file 'qhttprequest.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../dlna/dlnaHttpServer/qhttprequest.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qhttprequest.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12QHttpRequestE_t {};
} // unnamed namespace

template <> constexpr inline auto QHttpRequest::qt_create_metaobjectdata<qt_meta_tag_ZN12QHttpRequestE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QHttpRequest",
        "data",
        "",
        "end",
        "appendBody",
        "body",
        "headers",
        "HeaderHash",
        "remoteAddress",
        "remotePort",
        "method",
        "HttpMethod",
        "url",
        "QUrl",
        "path",
        "httpVersion",
        "HTTP_DELETE",
        "HTTP_GET",
        "HTTP_HEAD",
        "HTTP_POST",
        "HTTP_PUT",
        "HTTP_CONNECT",
        "HTTP_OPTIONS",
        "HTTP_TRACE",
        "HTTP_COPY",
        "HTTP_LOCK",
        "HTTP_MKCOL",
        "HTTP_MOVE",
        "HTTP_PROPFIND",
        "HTTP_PROPPATCH",
        "HTTP_SEARCH",
        "HTTP_UNLOCK",
        "HTTP_REPORT",
        "HTTP_MKACTIVITY",
        "HTTP_CHECKOUT",
        "HTTP_MERGE",
        "HTTP_MSEARCH",
        "HTTP_NOTIFY",
        "HTTP_SUBSCRIBE",
        "HTTP_UNSUBSCRIBE",
        "HTTP_PATCH",
        "HTTP_PURGE"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'data'
        QtMocHelpers::SignalData<void(const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 1 },
        }}),
        // Signal 'end'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'appendBody'
        QtMocHelpers::SlotData<void(const QByteArray &)>(4, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'headers'
        QtMocHelpers::PropertyData<HeaderHash>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'remoteAddress'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'remotePort'
        QtMocHelpers::PropertyData<quint16>(9, QMetaType::UShort, QMC::DefaultPropertyFlags),
        // property 'method'
        QtMocHelpers::PropertyData<enum HttpMethod>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'url'
        QtMocHelpers::PropertyData<QUrl>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'path'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'httpVersion'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'HttpMethod'
        QtMocHelpers::EnumData<enum HttpMethod>(11, 11, QMC::EnumFlags{}).add({
            {   16, HttpMethod::HTTP_DELETE },
            {   17, HttpMethod::HTTP_GET },
            {   18, HttpMethod::HTTP_HEAD },
            {   19, HttpMethod::HTTP_POST },
            {   20, HttpMethod::HTTP_PUT },
            {   21, HttpMethod::HTTP_CONNECT },
            {   22, HttpMethod::HTTP_OPTIONS },
            {   23, HttpMethod::HTTP_TRACE },
            {   24, HttpMethod::HTTP_COPY },
            {   25, HttpMethod::HTTP_LOCK },
            {   26, HttpMethod::HTTP_MKCOL },
            {   27, HttpMethod::HTTP_MOVE },
            {   28, HttpMethod::HTTP_PROPFIND },
            {   29, HttpMethod::HTTP_PROPPATCH },
            {   30, HttpMethod::HTTP_SEARCH },
            {   31, HttpMethod::HTTP_UNLOCK },
            {   32, HttpMethod::HTTP_REPORT },
            {   33, HttpMethod::HTTP_MKACTIVITY },
            {   34, HttpMethod::HTTP_CHECKOUT },
            {   35, HttpMethod::HTTP_MERGE },
            {   36, HttpMethod::HTTP_MSEARCH },
            {   37, HttpMethod::HTTP_NOTIFY },
            {   38, HttpMethod::HTTP_SUBSCRIBE },
            {   39, HttpMethod::HTTP_UNSUBSCRIBE },
            {   40, HttpMethod::HTTP_PATCH },
            {   41, HttpMethod::HTTP_PURGE },
        }),
    };
    return QtMocHelpers::metaObjectData<QHttpRequest, qt_meta_tag_ZN12QHttpRequestE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QHttpRequest::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QHttpRequestE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QHttpRequestE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12QHttpRequestE_t>.metaTypes,
    nullptr
} };

void QHttpRequest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QHttpRequest *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->data((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 1: _t->end(); break;
        case 2: _t->appendBody((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QHttpRequest::*)(const QByteArray & )>(_a, &QHttpRequest::data, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QHttpRequest::*)()>(_a, &QHttpRequest::end, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<HeaderHash*>(_v) = _t->headers(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->remoteAddress(); break;
        case 2: *reinterpret_cast<quint16*>(_v) = _t->remotePort(); break;
        case 3: *reinterpret_cast<enum HttpMethod*>(_v) = _t->method(); break;
        case 4: *reinterpret_cast<QUrl*>(_v) = _t->url(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->path(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->httpVersion(); break;
        default: break;
        }
    }
}

const QMetaObject *QHttpRequest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QHttpRequest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12QHttpRequestE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QHttpRequest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QHttpRequest::data(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QHttpRequest::end()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
