/****************************************************************************
** Meta object code from reading C++ file 'presenter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../vendor/presenter.h"
#include <QtGui/qtextcursor.h>
#include <QtGui/qscreen.h>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'presenter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9PresenterE_t {};
} // unnamed namespace

template <> constexpr inline auto Presenter::qt_create_metaobjectdata<qt_meta_tag_ZN9PresenterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Presenter",
        "slotplay",
        "",
        "slotpause",
        "slotplaynext",
        "slotplayprev",
        "slotvolumeRequested",
        "volume",
        "slotopenUrlRequested",
        "QUrl",
        "url",
        "slotstateChanged",
        "slotloopStatusRequested",
        "Mpris::LoopStatus",
        "loopStatus",
        "slotplayModeChanged",
        "PlaylistModel::PlayMode",
        "pm",
        "slotvolumeChanged",
        "slotseek",
        "Offset",
        "slotstop"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotplay'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotpause'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotplaynext'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotplayprev'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotvolumeRequested'
        QtMocHelpers::SlotData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'slotopenUrlRequested'
        QtMocHelpers::SlotData<void(const QUrl)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'slotstateChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotloopStatusRequested'
        QtMocHelpers::SlotData<void(Mpris::LoopStatus)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'slotplayModeChanged'
        QtMocHelpers::SlotData<void(PlaylistModel::PlayMode)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotvolumeChanged'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotseek'
        QtMocHelpers::SlotData<void(qlonglong)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 20 },
        }}),
        // Slot 'slotstop'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Presenter, qt_meta_tag_ZN9PresenterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Presenter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9PresenterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9PresenterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9PresenterE_t>.metaTypes,
    nullptr
} };

void Presenter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Presenter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotplay(); break;
        case 1: _t->slotpause(); break;
        case 2: _t->slotplaynext(); break;
        case 3: _t->slotplayprev(); break;
        case 4: _t->slotvolumeRequested((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 5: _t->slotopenUrlRequested((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 6: _t->slotstateChanged(); break;
        case 7: _t->slotloopStatusRequested((*reinterpret_cast<std::add_pointer_t<Mpris::LoopStatus>>(_a[1]))); break;
        case 8: _t->slotplayModeChanged((*reinterpret_cast<std::add_pointer_t<PlaylistModel::PlayMode>>(_a[1]))); break;
        case 9: _t->slotvolumeChanged(); break;
        case 10: _t->slotseek((*reinterpret_cast<std::add_pointer_t<qlonglong>>(_a[1]))); break;
        case 11: _t->slotstop(); break;
        default: ;
        }
    }
}

const QMetaObject *Presenter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Presenter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9PresenterE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Presenter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
