/****************************************************************************
** Meta object code from reading C++ file 'playlist_model.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../libdmr/playlist_model.h"
#include <QtGui/qtextcursor.h>
#include <QtGui/qscreen.h>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlist_model.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3dmr13PlaylistModelE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::PlaylistModel::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr13PlaylistModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::PlaylistModel",
        "countChanged",
        "",
        "currentChanged",
        "itemRemoved",
        "itemsAppended",
        "emptied",
        "playModeChanged",
        "PlayMode",
        "asyncAppendFinished",
        "QList<PlayItemInfo>",
        "itemInfoUpdated",
        "id",
        "updateDuration",
        "changeCurrent",
        "delayedAppendAsync",
        "QList<QUrl>",
        "clearLoad",
        "onAsyncFinished",
        "onAsyncUpdate",
        "PlayItemInfo",
        "slotStateChanged",
        "count",
        "current"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'itemRemoved'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'itemsAppended'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'emptied'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'playModeChanged'
        QtMocHelpers::SignalData<void(enum PlayMode)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'asyncAppendFinished'
        QtMocHelpers::SignalData<void(const QList<PlayItemInfo> &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Signal 'itemInfoUpdated'
        QtMocHelpers::SignalData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 12 },
        }}),
        // Signal 'updateDuration'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'changeCurrent'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'delayedAppendAsync'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 2 },
        }}),
        // Slot 'clearLoad'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onAsyncFinished'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onAsyncUpdate'
        QtMocHelpers::SlotData<void(PlayItemInfo)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 2 },
        }}),
        // Slot 'slotStateChanged'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'count'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
        // property 'current'
        QtMocHelpers::PropertyData<int>(23, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlaylistModel, qt_meta_tag_ZN3dmr13PlaylistModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::PlaylistModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr13PlaylistModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr13PlaylistModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr13PlaylistModelE_t>.metaTypes,
    nullptr
} };

void dmr::PlaylistModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->countChanged(); break;
        case 1: _t->currentChanged(); break;
        case 2: _t->itemRemoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->itemsAppended(); break;
        case 4: _t->emptied(); break;
        case 5: _t->playModeChanged((*reinterpret_cast<std::add_pointer_t<enum PlayMode>>(_a[1]))); break;
        case 6: _t->asyncAppendFinished((*reinterpret_cast<std::add_pointer_t<QList<PlayItemInfo>>>(_a[1]))); break;
        case 7: _t->itemInfoUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->updateDuration(); break;
        case 9: _t->changeCurrent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->delayedAppendAsync((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 11: _t->clearLoad(); break;
        case 12: _t->onAsyncFinished(); break;
        case 13: _t->onAsyncUpdate((*reinterpret_cast<std::add_pointer_t<PlayItemInfo>>(_a[1]))); break;
        case 14: _t->slotStateChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlaylistModel::*)()>(_a, &PlaylistModel::countChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistModel::*)()>(_a, &PlaylistModel::currentChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistModel::*)(int )>(_a, &PlaylistModel::itemRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistModel::*)()>(_a, &PlaylistModel::itemsAppended, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistModel::*)()>(_a, &PlaylistModel::emptied, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistModel::*)(PlayMode )>(_a, &PlaylistModel::playModeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistModel::*)(const QList<PlayItemInfo> & )>(_a, &PlaylistModel::asyncAppendFinished, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistModel::*)(int )>(_a, &PlaylistModel::itemInfoUpdated, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistModel::*)()>(_a, &PlaylistModel::updateDuration, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->count(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->current(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->changeCurrent(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *dmr::PlaylistModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::PlaylistModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr13PlaylistModelE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dmr::PlaylistModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void dmr::PlaylistModel::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dmr::PlaylistModel::currentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dmr::PlaylistModel::itemRemoved(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void dmr::PlaylistModel::itemsAppended()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void dmr::PlaylistModel::emptied()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void dmr::PlaylistModel::playModeChanged(PlayMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void dmr::PlaylistModel::asyncAppendFinished(const QList<PlayItemInfo> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void dmr::PlaylistModel::itemInfoUpdated(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void dmr::PlaylistModel::updateDuration()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
namespace {
struct qt_meta_tag_ZN3dmr10LoadThreadE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::LoadThread::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr10LoadThreadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::LoadThread"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LoadThread, qt_meta_tag_ZN3dmr10LoadThreadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::LoadThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr10LoadThreadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr10LoadThreadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr10LoadThreadE_t>.metaTypes,
    nullptr
} };

void dmr::LoadThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LoadThread *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *dmr::LoadThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::LoadThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr10LoadThreadE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int dmr::LoadThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN3dmr12GetThumanbilE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::GetThumanbil::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr12GetThumanbilE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::GetThumanbil",
        "updateItem",
        "",
        "PlayItemInfo"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updateItem'
        QtMocHelpers::SignalData<void(PlayItemInfo)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GetThumanbil, qt_meta_tag_ZN3dmr12GetThumanbilE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::GetThumanbil::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr12GetThumanbilE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr12GetThumanbilE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr12GetThumanbilE_t>.metaTypes,
    nullptr
} };

void dmr::GetThumanbil::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GetThumanbil *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateItem((*reinterpret_cast<std::add_pointer_t<PlayItemInfo>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GetThumanbil::*)(PlayItemInfo )>(_a, &GetThumanbil::updateItem, 0))
            return;
    }
}

const QMetaObject *dmr::GetThumanbil::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::GetThumanbil::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr12GetThumanbilE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int dmr::GetThumanbil::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void dmr::GetThumanbil::updateItem(PlayItemInfo _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
