/****************************************************************************
** Meta object code from reading C++ file 'actions.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../common/actions.h"
#include <QtGui/qtextcursor.h>
#include <QtGui/qscreen.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'actions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3dmr13ActionFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::ActionFactory::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr13ActionFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::ActionFactory",
        "frameMenuEnable",
        "",
        "playSpeedMenuEnable",
        "subtitleMenuEnable",
        "soundMenuEnable",
        "ActionKind",
        "Invalid",
        "OpenFile",
        "OpenFileList",
        "OpenDirectory",
        "StartPlay",
        "Settings",
        "About",
        "Help",
        "Exit",
        "TogglePlaylist",
        "EmptyPlaylist",
        "PlaylistRemoveItem",
        "PlaylistOpenItemInFM",
        "PlaylistItemInfo",
        "MovieInfo",
        "OpenUrl",
        "OpenCdrom",
        "ToggleFullscreen",
        "QuitFullscreen",
        "ToggleMiniMode",
        "WindowAbove",
        "LoadSubtitle",
        "SelectSubtitle",
        "HideSubtitle",
        "MatchOnlineSubtitle",
        "ChangeSubCodepage",
        "Screenshot",
        "BurstScreenshot",
        "GoToScreenshotSolder",
        "SeekForward",
        "SeekForwardLarge",
        "SeekBackward",
        "SeekBackwardLarge",
        "SeekAbsolute",
        "TogglePause",
        "Stop",
        "AccelPlayback",
        "DecelPlayback",
        "ResetPlayback",
        "SubDelay",
        "SubForward",
        "OrderPlay",
        "ShufflePlay",
        "SinglePlay",
        "SingleLoop",
        "ListLoop",
        "ZeroPointFiveTimes",
        "OneTimes",
        "OnePointTwoTimes",
        "OnePointFiveTimes",
        "Double",
        "DefaultFrame",
        "Ratio4x3Frame",
        "Ratio16x9Frame",
        "Ratio16x10Frame",
        "Ratio185x1Frame",
        "Ratio235x1Frame",
        "ClockwiseFrame",
        "CounterclockwiseFrame",
        "NextFrame",
        "PreviousFrame",
        "Stereo",
        "LeftChannel",
        "RightChannel",
        "LoadTrack",
        "SelectTrack",
        "GotoPlaylistNext",
        "GotoPlaylistPrev",
        "GotoPlaylistSelected",
        "VolumeUp",
        "VolumeDown",
        "ToggleMute",
        "ChangeVolume",
        "ViewShortcut",
        "Hwdec"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'frameMenuEnable'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'playSpeedMenuEnable'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'subtitleMenuEnable'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'soundMenuEnable'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ActionKind'
        QtMocHelpers::EnumData<enum ActionKind>(6, 6, QMC::EnumFlags{}).add({
            {    7, ActionKind::Invalid },
            {    8, ActionKind::OpenFile },
            {    9, ActionKind::OpenFileList },
            {   10, ActionKind::OpenDirectory },
            {   11, ActionKind::StartPlay },
            {   12, ActionKind::Settings },
            {   13, ActionKind::About },
            {   14, ActionKind::Help },
            {   15, ActionKind::Exit },
            {   16, ActionKind::TogglePlaylist },
            {   17, ActionKind::EmptyPlaylist },
            {   18, ActionKind::PlaylistRemoveItem },
            {   19, ActionKind::PlaylistOpenItemInFM },
            {   20, ActionKind::PlaylistItemInfo },
            {   21, ActionKind::MovieInfo },
            {   22, ActionKind::OpenUrl },
            {   23, ActionKind::OpenCdrom },
            {   24, ActionKind::ToggleFullscreen },
            {   25, ActionKind::QuitFullscreen },
            {   26, ActionKind::ToggleMiniMode },
            {   27, ActionKind::WindowAbove },
            {   28, ActionKind::LoadSubtitle },
            {   29, ActionKind::SelectSubtitle },
            {   30, ActionKind::HideSubtitle },
            {   31, ActionKind::MatchOnlineSubtitle },
            {   32, ActionKind::ChangeSubCodepage },
            {   33, ActionKind::Screenshot },
            {   34, ActionKind::BurstScreenshot },
            {   35, ActionKind::GoToScreenshotSolder },
            {   36, ActionKind::SeekForward },
            {   37, ActionKind::SeekForwardLarge },
            {   38, ActionKind::SeekBackward },
            {   39, ActionKind::SeekBackwardLarge },
            {   40, ActionKind::SeekAbsolute },
            {   41, ActionKind::TogglePause },
            {   42, ActionKind::Stop },
            {   43, ActionKind::AccelPlayback },
            {   44, ActionKind::DecelPlayback },
            {   45, ActionKind::ResetPlayback },
            {   46, ActionKind::SubDelay },
            {   47, ActionKind::SubForward },
            {   48, ActionKind::OrderPlay },
            {   49, ActionKind::ShufflePlay },
            {   50, ActionKind::SinglePlay },
            {   51, ActionKind::SingleLoop },
            {   52, ActionKind::ListLoop },
            {   53, ActionKind::ZeroPointFiveTimes },
            {   54, ActionKind::OneTimes },
            {   55, ActionKind::OnePointTwoTimes },
            {   56, ActionKind::OnePointFiveTimes },
            {   57, ActionKind::Double },
            {   58, ActionKind::DefaultFrame },
            {   59, ActionKind::Ratio4x3Frame },
            {   60, ActionKind::Ratio16x9Frame },
            {   61, ActionKind::Ratio16x10Frame },
            {   62, ActionKind::Ratio185x1Frame },
            {   63, ActionKind::Ratio235x1Frame },
            {   64, ActionKind::ClockwiseFrame },
            {   65, ActionKind::CounterclockwiseFrame },
            {   66, ActionKind::NextFrame },
            {   67, ActionKind::PreviousFrame },
            {   68, ActionKind::Stereo },
            {   69, ActionKind::LeftChannel },
            {   70, ActionKind::RightChannel },
            {   71, ActionKind::LoadTrack },
            {   72, ActionKind::SelectTrack },
            {   73, ActionKind::GotoPlaylistNext },
            {   74, ActionKind::GotoPlaylistPrev },
            {   75, ActionKind::GotoPlaylistSelected },
            {   76, ActionKind::VolumeUp },
            {   77, ActionKind::VolumeDown },
            {   78, ActionKind::ToggleMute },
            {   79, ActionKind::ChangeVolume },
            {   80, ActionKind::ViewShortcut },
            {   81, ActionKind::Hwdec },
        }),
    };
    return QtMocHelpers::metaObjectData<ActionFactory, qt_meta_tag_ZN3dmr13ActionFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::ActionFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr13ActionFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr13ActionFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr13ActionFactoryE_t>.metaTypes,
    nullptr
} };

void dmr::ActionFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ActionFactory *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->frameMenuEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->playSpeedMenuEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->subtitleMenuEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->soundMenuEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ActionFactory::*)(bool )>(_a, &ActionFactory::frameMenuEnable, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActionFactory::*)(bool )>(_a, &ActionFactory::playSpeedMenuEnable, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActionFactory::*)(bool )>(_a, &ActionFactory::subtitleMenuEnable, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ActionFactory::*)(bool )>(_a, &ActionFactory::soundMenuEnable, 3))
            return;
    }
}

const QMetaObject *dmr::ActionFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::ActionFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr13ActionFactoryE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int dmr::ActionFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void dmr::ActionFactory::frameMenuEnable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void dmr::ActionFactory::playSpeedMenuEnable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void dmr::ActionFactory::subtitleMenuEnable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void dmr::ActionFactory::soundMenuEnable(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
