/****************************************************************************
** Meta object code from reading C++ file 'toolbutton.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../widgets/toolbutton.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'toolbutton.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3dmr15ButtonBoxButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::ButtonBoxButton::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr15ButtonBoxButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::ButtonBoxButton",
        "entered",
        "",
        "leaved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'entered'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'leaved'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ButtonBoxButton, qt_meta_tag_ZN3dmr15ButtonBoxButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::ButtonBoxButton::staticMetaObject = { {
    QMetaObject::SuperData::link<DButtonBoxButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr15ButtonBoxButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr15ButtonBoxButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr15ButtonBoxButtonE_t>.metaTypes,
    nullptr
} };

void dmr::ButtonBoxButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ButtonBoxButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->entered(); break;
        case 1: _t->leaved(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ButtonBoxButton::*)()>(_a, &ButtonBoxButton::entered, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ButtonBoxButton::*)()>(_a, &ButtonBoxButton::leaved, 1))
            return;
    }
}

const QMetaObject *dmr::ButtonBoxButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::ButtonBoxButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr15ButtonBoxButtonE_t>.strings))
        return static_cast<void*>(this);
    return DButtonBoxButton::qt_metacast(_clname);
}

int dmr::ButtonBoxButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DButtonBoxButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void dmr::ButtonBoxButton::entered()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dmr::ButtonBoxButton::leaved()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN3dmr13ButtonToolTipE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::ButtonToolTip::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr13ButtonToolTipE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::ButtonToolTip"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ButtonToolTip, qt_meta_tag_ZN3dmr13ButtonToolTipE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::ButtonToolTip::staticMetaObject = { {
    QMetaObject::SuperData::link<DArrowRectangle::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr13ButtonToolTipE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr13ButtonToolTipE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr13ButtonToolTipE_t>.metaTypes,
    nullptr
} };

void dmr::ButtonToolTip::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ButtonToolTip *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *dmr::ButtonToolTip::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::ButtonToolTip::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr13ButtonToolTipE_t>.strings))
        return static_cast<void*>(this);
    return DArrowRectangle::qt_metacast(_clname);
}

int dmr::ButtonToolTip::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DArrowRectangle::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN3dmr7ToolTipE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::ToolTip::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr7ToolTipE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::ToolTip",
        "slotWMChanged",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotWMChanged'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ToolTip, qt_meta_tag_ZN3dmr7ToolTipE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::ToolTip::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr7ToolTipE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr7ToolTipE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr7ToolTipE_t>.metaTypes,
    nullptr
} };

void dmr::ToolTip::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ToolTip *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotWMChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *dmr::ToolTip::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::ToolTip::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr7ToolTipE_t>.strings))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int dmr::ToolTip::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN3dmr10ToolButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::ToolButton::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr10ToolButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::ToolButton",
        "entered",
        "",
        "leaved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'entered'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'leaved'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ToolButton, qt_meta_tag_ZN3dmr10ToolButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::ToolButton::staticMetaObject = { {
    QMetaObject::SuperData::link<DIconButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr10ToolButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr10ToolButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr10ToolButtonE_t>.metaTypes,
    nullptr
} };

void dmr::ToolButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ToolButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->entered(); break;
        case 1: _t->leaved(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ToolButton::*)()>(_a, &ToolButton::entered, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ToolButton::*)()>(_a, &ToolButton::leaved, 1))
            return;
    }
}

const QMetaObject *dmr::ToolButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::ToolButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr10ToolButtonE_t>.strings))
        return static_cast<void*>(this);
    return DIconButton::qt_metacast(_clname);
}

int dmr::ToolButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DIconButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void dmr::ToolButton::entered()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dmr::ToolButton::leaved()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN3dmr12VolumeButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::VolumeButton::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr12VolumeButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::VolumeButton",
        "entered",
        "",
        "leaved",
        "requestVolumeUp",
        "requestVolumeDown",
        "sigUnsupported"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'entered'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'leaved'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestVolumeUp'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestVolumeDown'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sigUnsupported'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VolumeButton, qt_meta_tag_ZN3dmr12VolumeButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::VolumeButton::staticMetaObject = { {
    QMetaObject::SuperData::link<DIconButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr12VolumeButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr12VolumeButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr12VolumeButtonE_t>.metaTypes,
    nullptr
} };

void dmr::VolumeButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VolumeButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->entered(); break;
        case 1: _t->leaved(); break;
        case 2: _t->requestVolumeUp(); break;
        case 3: _t->requestVolumeDown(); break;
        case 4: _t->sigUnsupported(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (VolumeButton::*)()>(_a, &VolumeButton::entered, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (VolumeButton::*)()>(_a, &VolumeButton::leaved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (VolumeButton::*)()>(_a, &VolumeButton::requestVolumeUp, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (VolumeButton::*)()>(_a, &VolumeButton::requestVolumeDown, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (VolumeButton::*)()>(_a, &VolumeButton::sigUnsupported, 4))
            return;
    }
}

const QMetaObject *dmr::VolumeButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::VolumeButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr12VolumeButtonE_t>.strings))
        return static_cast<void*>(this);
    return DIconButton::qt_metacast(_clname);
}

int dmr::VolumeButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DIconButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dmr::VolumeButton::entered()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dmr::VolumeButton::leaved()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dmr::VolumeButton::requestVolumeUp()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void dmr::VolumeButton::requestVolumeDown()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void dmr::VolumeButton::sigUnsupported()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
