/****************************************************************************
** Meta object code from reading C++ file 'player_engine.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../libdmr/player_engine.h"
#include <QtGui/qtextcursor.h>
#include <QtGui/qscreen.h>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'player_engine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3dmr12PlayerEngineE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::PlayerEngine::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr12PlayerEngineE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::PlayerEngine",
        "tracksChanged",
        "",
        "elapsedChanged",
        "videoSizeChanged",
        "stateChanged",
        "fileLoaded",
        "muteChanged",
        "volumeChanged",
        "sidChanged",
        "aidChanged",
        "subCodepageChanged",
        "loadOnlineSubtitlesFinished",
        "QUrl",
        "url",
        "success",
        "mpvFunsLoadOver",
        "notifyScreenshot",
        "QImage",
        "frame",
        "time",
        "playlistChanged",
        "onlineStateChanged",
        "isOnline",
        "mpvErrorLogsChanged",
        "prefix",
        "text",
        "mpvWarningLogsChanged",
        "urlpause",
        "status",
        "siginitthumbnailseting",
        "updateDuration",
        "sigInvalidFile",
        "strFileName",
        "sigMediaError",
        "finishedAddFiles",
        "QList<QUrl>",
        "play",
        "pauseResume",
        "stop",
        "prev",
        "next",
        "playSelected",
        "id",
        "playByName",
        "clearPlaylist",
        "seekForward",
        "secs",
        "seekBackward",
        "seekAbsolute",
        "pos",
        "volumeUp",
        "volumeDown",
        "changeVolume",
        "val",
        "toggleMute",
        "setMute",
        "bMute",
        "onBackendStateChanged",
        "requestPlay",
        "updateSubStyles",
        "onSubtitlesDownloaded",
        "filenames",
        "OnlineSubtitle::FailReason",
        "onPlaylistAsyncAppendFinished",
        "QList<PlayItemInfo>",
        "duration",
        "elapsed",
        "videoSize",
        "QSize",
        "paused",
        "state",
        "CoreState",
        "Idle",
        "Playing",
        "Paused"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'tracksChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'elapsedChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'videoSizeChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fileLoaded'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'muteChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'volumeChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sidChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aidChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subCodepageChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loadOnlineSubtitlesFinished'
        QtMocHelpers::SignalData<void(const QUrl &, bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::Bool, 15 },
        }}),
        // Signal 'mpvFunsLoadOver'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notifyScreenshot'
        QtMocHelpers::SignalData<void(const QImage &, qint64)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { QMetaType::LongLong, 20 },
        }}),
        // Signal 'playlistChanged'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'onlineStateChanged'
        QtMocHelpers::SignalData<void(const bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Signal 'mpvErrorLogsChanged'
        QtMocHelpers::SignalData<void(const QString, const QString)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { QMetaType::QString, 26 },
        }}),
        // Signal 'mpvWarningLogsChanged'
        QtMocHelpers::SignalData<void(const QString, const QString)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { QMetaType::QString, 26 },
        }}),
        // Signal 'urlpause'
        QtMocHelpers::SignalData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Signal 'siginitthumbnailseting'
        QtMocHelpers::SignalData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateDuration'
        QtMocHelpers::SignalData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sigInvalidFile'
        QtMocHelpers::SignalData<void(QString)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Signal 'sigMediaError'
        QtMocHelpers::SignalData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finishedAddFiles'
        QtMocHelpers::SignalData<void(QList<QUrl>)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 2 },
        }}),
        // Slot 'play'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pauseResume'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'prev'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'next'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'playSelected'
        QtMocHelpers::SlotData<void(int)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 43 },
        }}),
        // Slot 'playByName'
        QtMocHelpers::SlotData<void(const QUrl &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'clearPlaylist'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'seekForward'
        QtMocHelpers::SlotData<void(int)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 47 },
        }}),
        // Slot 'seekBackward'
        QtMocHelpers::SlotData<void(int)>(48, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 47 },
        }}),
        // Slot 'seekAbsolute'
        QtMocHelpers::SlotData<void(int)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 50 },
        }}),
        // Slot 'volumeUp'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'volumeDown'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'changeVolume'
        QtMocHelpers::SlotData<void(int)>(53, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 54 },
        }}),
        // Slot 'toggleMute'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setMute'
        QtMocHelpers::SlotData<void(bool)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 57 },
        }}),
        // Slot 'onBackendStateChanged'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'requestPlay'
        QtMocHelpers::SlotData<void(int)>(59, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 43 },
        }}),
        // Slot 'updateSubStyles'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onSubtitlesDownloaded'
        QtMocHelpers::SlotData<void(const QUrl &, const QList<QString> &, OnlineSubtitle::FailReason)>(61, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QStringList, 62 }, { 0x80000000 | 63, 2 },
        }}),
        // Slot 'onPlaylistAsyncAppendFinished'
        QtMocHelpers::SlotData<void(const QList<PlayItemInfo> &)>(64, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 65, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'duration'
        QtMocHelpers::PropertyData<qint64>(66, QMetaType::LongLong, QMC::DefaultPropertyFlags),
        // property 'elapsed'
        QtMocHelpers::PropertyData<qint64>(67, QMetaType::LongLong, QMC::DefaultPropertyFlags, 1),
        // property 'videoSize'
        QtMocHelpers::PropertyData<QSize>(68, 0x80000000 | 69, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'paused'
        QtMocHelpers::PropertyData<bool>(70, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'state'
        QtMocHelpers::PropertyData<enum CoreState>(71, 0x80000000 | 72, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CoreState'
        QtMocHelpers::EnumData<enum CoreState>(72, 72, QMC::EnumFlags{}).add({
            {   73, CoreState::Idle },
            {   74, CoreState::Playing },
            {   75, CoreState::Paused },
        }),
    };
    return QtMocHelpers::metaObjectData<PlayerEngine, qt_meta_tag_ZN3dmr12PlayerEngineE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::PlayerEngine::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr12PlayerEngineE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr12PlayerEngineE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr12PlayerEngineE_t>.metaTypes,
    nullptr
} };

void dmr::PlayerEngine::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlayerEngine *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->tracksChanged(); break;
        case 1: _t->elapsedChanged(); break;
        case 2: _t->videoSizeChanged(); break;
        case 3: _t->stateChanged(); break;
        case 4: _t->fileLoaded(); break;
        case 5: _t->muteChanged(); break;
        case 6: _t->volumeChanged(); break;
        case 7: _t->sidChanged(); break;
        case 8: _t->aidChanged(); break;
        case 9: _t->subCodepageChanged(); break;
        case 10: _t->loadOnlineSubtitlesFinished((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->mpvFunsLoadOver(); break;
        case 12: _t->notifyScreenshot((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 13: _t->playlistChanged(); break;
        case 14: _t->onlineStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->mpvErrorLogsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 16: _t->mpvWarningLogsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 17: _t->urlpause((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->siginitthumbnailseting(); break;
        case 19: _t->updateDuration(); break;
        case 20: _t->sigInvalidFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->sigMediaError(); break;
        case 22: _t->finishedAddFiles((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 23: _t->play(); break;
        case 24: _t->pauseResume(); break;
        case 25: _t->stop(); break;
        case 26: _t->prev(); break;
        case 27: _t->next(); break;
        case 28: _t->playSelected((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->playByName((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 30: _t->clearPlaylist(); break;
        case 31: _t->seekForward((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 32: _t->seekBackward((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 33: _t->seekAbsolute((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 34: _t->volumeUp(); break;
        case 35: _t->volumeDown(); break;
        case 36: _t->changeVolume((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 37: _t->toggleMute(); break;
        case 38: _t->setMute((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: _t->onBackendStateChanged(); break;
        case 40: _t->requestPlay((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 41: _t->updateSubStyles(); break;
        case 42: _t->onSubtitlesDownloaded((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QString>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<OnlineSubtitle::FailReason>>(_a[3]))); break;
        case 43: _t->onPlaylistAsyncAppendFinished((*reinterpret_cast<std::add_pointer_t<QList<PlayItemInfo>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::tracksChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::elapsedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::videoSizeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::stateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::fileLoaded, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::muteChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::volumeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::sidChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::aidChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::subCodepageChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)(const QUrl & , bool )>(_a, &PlayerEngine::loadOnlineSubtitlesFinished, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::mpvFunsLoadOver, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)(const QImage & , qint64 )>(_a, &PlayerEngine::notifyScreenshot, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::playlistChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)(const bool )>(_a, &PlayerEngine::onlineStateChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)(const QString , const QString )>(_a, &PlayerEngine::mpvErrorLogsChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)(const QString , const QString )>(_a, &PlayerEngine::mpvWarningLogsChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)(bool )>(_a, &PlayerEngine::urlpause, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::siginitthumbnailseting, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::updateDuration, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)(QString )>(_a, &PlayerEngine::sigInvalidFile, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)()>(_a, &PlayerEngine::sigMediaError, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlayerEngine::*)(QList<QUrl> )>(_a, &PlayerEngine::finishedAddFiles, 22))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qint64*>(_v) = _t->duration(); break;
        case 1: *reinterpret_cast<qint64*>(_v) = _t->elapsed(); break;
        case 2: *reinterpret_cast<QSize*>(_v) = _t->videoSize(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->paused(); break;
        case 4: *reinterpret_cast<enum CoreState*>(_v) = _t->state(); break;
        default: break;
        }
    }
}

const QMetaObject *dmr::PlayerEngine::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::PlayerEngine::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr12PlayerEngineE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int dmr::PlayerEngine::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 44)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 44;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dmr::PlayerEngine::tracksChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dmr::PlayerEngine::elapsedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dmr::PlayerEngine::videoSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void dmr::PlayerEngine::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void dmr::PlayerEngine::fileLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void dmr::PlayerEngine::muteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void dmr::PlayerEngine::volumeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void dmr::PlayerEngine::sidChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void dmr::PlayerEngine::aidChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void dmr::PlayerEngine::subCodepageChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void dmr::PlayerEngine::loadOnlineSubtitlesFinished(const QUrl & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void dmr::PlayerEngine::mpvFunsLoadOver()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void dmr::PlayerEngine::notifyScreenshot(const QImage & _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void dmr::PlayerEngine::playlistChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void dmr::PlayerEngine::onlineStateChanged(const bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void dmr::PlayerEngine::mpvErrorLogsChanged(const QString _t1, const QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1, _t2);
}

// SIGNAL 16
void dmr::PlayerEngine::mpvWarningLogsChanged(const QString _t1, const QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void dmr::PlayerEngine::urlpause(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void dmr::PlayerEngine::siginitthumbnailseting()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void dmr::PlayerEngine::updateDuration()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void dmr::PlayerEngine::sigInvalidFile(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void dmr::PlayerEngine::sigMediaError()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void dmr::PlayerEngine::finishedAddFiles(QList<QUrl> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}
QT_WARNING_POP
