/****************************************************************************
** Meta object code from reading C++ file 'player_backend.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../libdmr/player_backend.h"
#include <QtGui/qtextcursor.h>
#include <QtGui/qscreen.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'player_backend.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3dmr7BackendE_t {};
} // unnamed namespace

template <> constexpr inline auto dmr::Backend::qt_create_metaobjectdata<qt_meta_tag_ZN3dmr7BackendE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dmr::Backend",
        "tracksChanged",
        "",
        "elapsedChanged",
        "videoSizeChanged",
        "stateChanged",
        "fileLoaded",
        "muteChanged",
        "volumeChanged",
        "sidChanged",
        "aidChanged",
        "notifyScreenshot",
        "QImage",
        "frame",
        "time",
        "mpvErrorLogsChanged",
        "prefix",
        "text",
        "mpvWarningLogsChanged",
        "urlpause",
        "status",
        "sigMediaError",
        "play",
        "pauseResume",
        "stop",
        "seekForward",
        "secs",
        "seekBackward",
        "seekAbsolute",
        "volumeUp",
        "volumeDown",
        "changeVolume",
        "val",
        "toggleMute",
        "setMute",
        "bMute",
        "duration",
        "elapsed",
        "videoSize",
        "QSize",
        "paused",
        "state",
        "PlayState",
        "Playing",
        "Paused",
        "Stopped",
        "DebugLevel",
        "Info",
        "Debug",
        "Verbose",
        "SoundMode",
        "Stereo",
        "Left",
        "Right",
        "hwaccelMode",
        "hwaccelAuto",
        "hwaccelOpen",
        "hwaccelClose"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'tracksChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'elapsedChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'videoSizeChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fileLoaded'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'muteChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'volumeChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sidChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aidChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notifyScreenshot'
        QtMocHelpers::SignalData<void(const QImage &, qint64)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::LongLong, 14 },
        }}),
        // Signal 'mpvErrorLogsChanged'
        QtMocHelpers::SignalData<void(const QString, const QString)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 },
        }}),
        // Signal 'mpvWarningLogsChanged'
        QtMocHelpers::SignalData<void(const QString, const QString)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 },
        }}),
        // Signal 'urlpause'
        QtMocHelpers::SignalData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Signal 'sigMediaError'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'play'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pauseResume'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'seekForward'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Slot 'seekBackward'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Slot 'seekAbsolute'
        QtMocHelpers::SlotData<void(int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'volumeUp'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'volumeDown'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'changeVolume'
        QtMocHelpers::SlotData<void(int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 },
        }}),
        // Slot 'toggleMute'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setMute'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'duration'
        QtMocHelpers::PropertyData<qint64>(36, QMetaType::LongLong, QMC::DefaultPropertyFlags),
        // property 'elapsed'
        QtMocHelpers::PropertyData<qint64>(37, QMetaType::LongLong, QMC::DefaultPropertyFlags, 1),
        // property 'videoSize'
        QtMocHelpers::PropertyData<QSize>(38, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'paused'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'state'
        QtMocHelpers::PropertyData<enum PlayState>(41, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PlayState'
        QtMocHelpers::EnumData<enum PlayState>(42, 42, QMC::EnumFlags{}).add({
            {   43, PlayState::Playing },
            {   44, PlayState::Paused },
            {   45, PlayState::Stopped },
        }),
        // enum 'DebugLevel'
        QtMocHelpers::EnumData<enum DebugLevel>(46, 46, QMC::EnumFlags{}).add({
            {   47, DebugLevel::Info },
            {   48, DebugLevel::Debug },
            {   49, DebugLevel::Verbose },
        }),
        // enum 'SoundMode'
        QtMocHelpers::EnumData<enum SoundMode>(50, 50, QMC::EnumFlags{}).add({
            {   51, SoundMode::Stereo },
            {   52, SoundMode::Left },
            {   53, SoundMode::Right },
        }),
        // enum 'hwaccelMode'
        QtMocHelpers::EnumData<enum hwaccelMode>(54, 54, QMC::EnumFlags{}).add({
            {   55, hwaccelMode::hwaccelAuto },
            {   56, hwaccelMode::hwaccelOpen },
            {   57, hwaccelMode::hwaccelClose },
        }),
    };
    return QtMocHelpers::metaObjectData<Backend, qt_meta_tag_ZN3dmr7BackendE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dmr::Backend::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr7BackendE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr7BackendE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dmr7BackendE_t>.metaTypes,
    nullptr
} };

void dmr::Backend::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Backend *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->tracksChanged(); break;
        case 1: _t->elapsedChanged(); break;
        case 2: _t->videoSizeChanged(); break;
        case 3: _t->stateChanged(); break;
        case 4: _t->fileLoaded(); break;
        case 5: _t->muteChanged(); break;
        case 6: _t->volumeChanged(); break;
        case 7: _t->sidChanged(); break;
        case 8: _t->aidChanged(); break;
        case 9: _t->notifyScreenshot((*reinterpret_cast<std::add_pointer_t<QImage>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2]))); break;
        case 10: _t->mpvErrorLogsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 11: _t->mpvWarningLogsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 12: _t->urlpause((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->sigMediaError(); break;
        case 14: _t->play(); break;
        case 15: _t->pauseResume(); break;
        case 16: _t->stop(); break;
        case 17: _t->seekForward((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->seekBackward((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->seekAbsolute((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->volumeUp(); break;
        case 21: _t->volumeDown(); break;
        case 22: _t->changeVolume((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->toggleMute(); break;
        case 24: _t->setMute((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Backend::*)()>(_a, &Backend::tracksChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)()>(_a, &Backend::elapsedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)()>(_a, &Backend::videoSizeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)()>(_a, &Backend::stateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)()>(_a, &Backend::fileLoaded, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)()>(_a, &Backend::muteChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)()>(_a, &Backend::volumeChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)()>(_a, &Backend::sidChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)()>(_a, &Backend::aidChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)(const QImage & , qint64 )>(_a, &Backend::notifyScreenshot, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)(const QString , const QString )>(_a, &Backend::mpvErrorLogsChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)(const QString , const QString )>(_a, &Backend::mpvWarningLogsChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)(bool )>(_a, &Backend::urlpause, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Backend::*)()>(_a, &Backend::sigMediaError, 13))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qint64*>(_v) = _t->duration(); break;
        case 1: *reinterpret_cast<qint64*>(_v) = _t->elapsed(); break;
        case 2: *reinterpret_cast<QSize*>(_v) = _t->videoSize(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->paused(); break;
        case 4: *reinterpret_cast<enum PlayState*>(_v) = _t->state(); break;
        default: break;
        }
    }
}

const QMetaObject *dmr::Backend::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dmr::Backend::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dmr7BackendE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int dmr::Backend::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void dmr::Backend::tracksChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void dmr::Backend::elapsedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void dmr::Backend::videoSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void dmr::Backend::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void dmr::Backend::fileLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void dmr::Backend::muteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void dmr::Backend::volumeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void dmr::Backend::sidChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void dmr::Backend::aidChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void dmr::Backend::notifyScreenshot(const QImage & _t1, qint64 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void dmr::Backend::mpvErrorLogsChanged(const QString _t1, const QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void dmr::Backend::mpvWarningLogsChanged(const QString _t1, const QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void dmr::Backend::urlpause(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void dmr::Backend::sigMediaError()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}
QT_WARNING_POP
